package pl.alx.kurs.colourlovers.model;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;

import pl.alx.kurs.utils.InputStreamTemplate;
import android.graphics.BitmapFactory;

public class PatternsModel {
	private static XmlPullParserFactory parserFactory;
	private PatternsListener listener;
	private static Executor egzekutor = Executors.newFixedThreadPool(3);
	
	static {
		try {
			parserFactory = XmlPullParserFactory.newInstance();
		} catch (XmlPullParserException e) {
			throw new RuntimeException(e);
		}		
	}
	
	private List<Pattern> patterns = Collections.synchronizedList(new ArrayList<Pattern>());

	private void loadPatterns() {
		new InputStreamTemplate() {
			@Override
			public InputStream createInputStream() throws Exception {
				URL url = new URL("http://www.colourlovers.com/api/patterns");
				HttpURLConnection huc = (HttpURLConnection) url.openConnection();
				return huc.getInputStream();
			}
			@Override
			public void useInputStream(InputStream is) throws Exception {
				Reader r = new InputStreamReader(is, "UTF-8");
				BufferedReader br = new BufferedReader(r, 1024 * 30);
				XmlPullParser xpp = parserFactory.newPullParser();
				xpp.setInput(br);
				Pattern pattern = new Pattern();
				SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
				while (xpp.getEventType() != XmlPullParser.END_DOCUMENT) {
					xpp.next();
					String name = xpp.getName();
					int type = xpp.getEventType();
					if (type == XmlPullParser.START_TAG) {
						if (name.equals("pattern")) {
							pattern = new Pattern();
						} else if (name.equals("id")) {
							pattern.setId(Long.parseLong(xpp.nextText()));
						} else if (name.equals("title")) {
							pattern.setTitle(xpp.nextText());
						} else if (name.equals("dateCreated")) {
							pattern.setDateCreated(dateFormat.parse(xpp.nextText()));
						} else if (name.equals("url")) {
							pattern.setUrl(xpp.nextText());
						} else if (name.equals("imageUrl")) {
							pattern.setImageUrl(xpp.nextText());
						}
					} else if (type == XmlPullParser.END_TAG) {
						if (name.equals("pattern")) {
							patterns.add(pattern);
						}
					}
				}
			}
		}.execute();
	}

	private void loadImage(final Pattern p) {
		new InputStreamTemplate() {
			@Override
			public void useInputStream(InputStream is) throws Exception {
				p.setImage(BitmapFactory.decodeStream(is));
			}
			@Override
			public InputStream createInputStream() throws Exception {
				return new URL(p.getImageUrl()).openStream();
			}
		}.execute();
	}

//	public void loadAsync() {
//		new Thread(new Runnable() {
//			public void run() {
//				loadPatterns();
//				if (listener != null) listener.patternsHaveChanged();
//				for (final Pattern p : patterns) {
//					new Thread(new Runnable() {
//						public void run() {
//							loadImage(p);
//							if (listener != null) listener.patternsHaveChanged();
//						}
//					}).start();
//				}
//			}
//		}).start();
//	}

	public void loadAsync() {
		egzekutor.execute(new Runnable() {
			public void run() {
				loadPatterns();
				if (listener != null) listener.patternsHaveChanged();
				for (final Pattern p : patterns) {
					egzekutor.execute(new Runnable() {
						public void run() {
							loadImage(p);
							if (listener != null) listener.patternsHaveChanged();
						}
					});
				}
			}
		});
	}	
	
	public List<Pattern> getPatterns() {
		return patterns;
	}

	public void setListener(PatternsListener listener) {
		this.listener = listener;
	}

	public void setPatterns(List<Pattern> patterns) {
		this.patterns = patterns;
	}
	
}
